==> bibledit-0.7.tar.gz: bibledit-0.7/src/usfmtools.cpp <==
/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"


ustring usfm_extract (ustring & line)
{
  /*
     This returns the usfm from the line, e.g. \id.
     The usfm is removed from the line.
   */
  ustring s = trim (line);
  ustring returnvalue = "";
  if (s.length () > 1)
    {
      if (s.substr (0, 1) == "\\")
        {
          unsigned int endposition = s.find_first_of (" \\", 1);
          if (endposition == string::npos)
            {
              line = "";
              returnvalue = s;
            }
          else
            {
              returnvalue = s.substr (0, endposition);
              line.erase (0, endposition);
              line = trim (line);
            }
        }
    }
  return returnvalue;
}


ustring usfm_extract_marker (ustring & line)
{
  // Returns the usfm marker from the line, but without
  // the first backslash, so e.g. id.
  // The backslash and the usfm marker are removed
  // from the line.
  ustring returnvalue = usfm_extract (line);
  if (returnvalue.length () > 0)
    returnvalue.erase (0, 1);   // Remove backslash.
  return returnvalue;
}


int
usfm_search_marker_from (const ustring & line, unsigned int from)
{
  /* 
     Searches for any usfm (whether it exists or not).
     It starts searching at position "from".
     It returns the position where it found the usfm, or
     string::npos is not found.
   */
  return line.find ("\\", from);
}


unsigned int
usfm_search_marker_from_length (const ustring & line, unsigned int from)
{
  /*
     Returns the length of the usfm that is found at position
     "from".
   */
  ustring s;
  s = line.substr (from, line.length ());
  return usfm_extract (s).length ();
}


string
get_id_from_file (const string & filename)
{
  ReadText rt (filename, true);
  unsigned int max = 10;
  if (max >= rt.lines.size ())
    max = rt.lines.size ();
  Usfm usfm (0);
  for (unsigned int i = 0; i < max; i++)
    {
      ustring line = trim (rt.lines[i]);
      ustring marker = usfm_extract_marker (line);
      if (usfm.is_id (marker))
        {
          try
          {
            return line.substr (0, 3);
          }
          catch (exception & ex)
          {
            cerr << ex.what () << endl;
          }
        }
    }
  return "";
}


void split_line_into_usfm_and_text (const ustring& line, ustring& usfm, ustring& text)
/*
Splits all text into usfms and text. 
E.g. "\v 1 In the beginning ..." is split into:
- \v 1 
- In the beginning ...
*/
{
  // Do the first split.
  text = line;
  usfm = usfm_extract_marker (text);
  // If this line did not have any usfm, we're through.
  if (usfm.empty())
    return;
  // Certain usfms, like for chapter and verse, need to have the numbers too.
  Usfm usfm2(0);
  if (usfm2.is_chapter (usfm) || usfm2.is_verse (usfm)) {
    string number;
    number = number_in_string (text);
    text.erase (0, number.length());
    text = trim (text);
    usfm.append (" ");
    usfm.append (number);
  }
  // The backslash was extracted, so put it back.
  usfm.insert(0, "\\");
}

==> bibledit-0.9.tar.gz: bibledit-0.9/src/usfmtools.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"


ustring usfm_extract (ustring & line)
{
  /*
     This returns the usfm from the line, e.g. \id.
     The usfm is removed from the line.
   */
  ustring s = trim (line);
  ustring returnvalue = "";
  if (s.length () > 1)
    {
      if (s.substr (0, 1) == "\\")
        {
          unsigned int endposition = s.find_first_of (" \\", 1);
          if (endposition == string::npos)
            {
              line = "";
              returnvalue = s;
            }
          else
            {
              returnvalue = s.substr (0, endposition);
              line.erase (0, endposition);
              line = trim (line);
            }
        }
    }
  return returnvalue;
}


ustring usfm_extract_marker (ustring & line)
{
  // Returns the usfm marker from the line, but without
  // the first backslash, so e.g. id.
  // The backslash and the usfm marker are removed
  // from the line.
  ustring returnvalue = usfm_extract (line);
  if (returnvalue.length () > 0)
    returnvalue.erase (0, 1);   // Remove backslash.
  return returnvalue;
}


int
usfm_search_marker_from (const ustring & line, unsigned int from)
{
  /* 
     Searches for any usfm (whether it exists or not).
     It starts searching at position "from".
     It returns the position where it found the usfm, or
     string::npos is not found.
   */
  return line.find ("\\", from);
}


unsigned int
usfm_search_marker_from_length (const ustring & line, unsigned int from)
{
  /*
     Returns the length of the usfm that is found at position
     "from".
   */
  ustring s;
  s = line.substr (from, line.length ());
  return usfm_extract (s).length ();
}


string
get_id_from_file (const string & filename, Usfm& usfm)
{
  ReadText rt (filename, true);
  unsigned int max = 10;
  if (max >= rt.lines.size ())
    max = rt.lines.size ();
  for (unsigned int i = 0; i < max; i++) {
    ustring line = trim (rt.lines[i]);
    ustring marker = usfm_extract_marker (line);
    if (usfm.is_identifier (marker)) {
      try 
      {
        return line.substr (0, 3);
      }
      catch (exception & ex)
      {
        cerr << ex.what () << endl;
      }
    }
  }
  return "";
}


void split_line_into_usfm_and_text (Usfm * usfm, const ustring& line, ustring& marker, ustring& text)
/*
Splits all text into marker and text. 
E.g. "\v 1 In the beginning ..." is split into:
- \v 1 
- In the beginning ...
*/
{
  // Do the first split.
  text = line;
  marker = usfm_extract_marker (text);
  // If this line did not have any usfm, we're through.
  if (marker.empty())
    return;
  // Certain usfms, like for chapter and verse, need to have the numbers too.
  if (usfm->is_chapter_number (marker) || usfm->is_verse_number (marker)) {
    string number;
    number = number_in_string (text);
    text.erase (0, number.length());
    text = trim (text);
    marker.append (" ");
    marker.append (number);
  }
  // The backslash was extracted, so put it back.
  marker.insert(0, "\\");
}
